@@ -9,14 +9,12 @@ use slings::AsyncWriteExt;
 fn main() -> io::Result<()> {
     slings::block_on(async {
         let addr: SocketAddr = "127.0.0.1:8080".parse().unwrap();
-        let listener = TcpListener::bind(addr).await.unwrap();
+        let listener = TcpListener::bind(addr).await?;
         println!("server start listen on 127.0.0.1:8080");
-
         loop {
-            let (mut stream, addr) = listener.accept().await.unwrap();
+            let (mut stream, addr) = listener.accept().await?;
             println!("accept stream from addr: {:?}", addr);
-
-            let task = slings::spawn_local(async move {
+            slings::spawn_local(async move {
                 loop {
                     let buf = b"helloworld";
                     match stream.write_all(&buf[..]).await {
@@ -30,9 +28,8 @@ fn main() -> io::Result<()> {
                     }
                     delay_for(Duration::from_secs(1)).await;
                 }
-            });
-            task.detach();
+            })
+            .detach();
         }
-    });
-    Ok(())
+    })
 }
